/**
 * KMuddy scripting - variable support
 *
 * #include this file in a C/C++ script, if you want to use variables
 * in that script.
 * 
 * */

//PUBLIC INTERFACE:

/** call this at the beginning of your script */
void initVariableSocket ();
/** call this at the end of your script */
void closeVariableSocket ();
/** reads variable value and store it in the buffer (of size buflen+1) */
void getVariable (const char *name, char *buffer, int buflen);
/** sets a variable value */
char setVariable (const char *name, const char *value);
/** unsets a variable */
char unsetVariable (const char *name);
/** increases a variable value */
char incVariable (const char *name, int delta);
/** decreases a variable value */
char decVariable (const char *name, int delta);
/** provide a resource */
char provideResource (const char *name);
/** request a resource */
char requestResource (const char *name);
/** request variable lock */
char lockVariable (const char *name);
/** unlock a variable */
void unlockVariable (const char *name);

//INTERNAL VARIABLES AND FUNCTIONS -- DO NOT USE IN THE SCRIPT
char internalBuffer[51];
int variableSocket = -1;
char simpleCommand (const char *command, const char *name);

void sendAndReceive (const char *sendthis, char *buffer, int buflen);

//IMPLEMENTATION

#include <errno.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/un.h>

void initVariableSocket ()
{
	struct sockaddr_un sa;
  char *path;
	path = getenv ("KMUDDY_SOCKET");
	if (!path)
	{
		puts ("Variable KMUDDY_SOCKET is not set - variable support won't work!\n");
		fflush (stdout);
		return; //bah!
	}
	variableSocket = socket (PF_UNIX, SOCK_STREAM, 0);
	sa.sun_family = AF_UNIX;
	strcpy (sa.sun_path, path);
	if (connect (variableSocket, (struct sockaddr *) &sa, sizeof (sa)) == -1)
	{
		close (variableSocket);
		variableSocket = -1;  // :(
		puts ("Could not connect to KMuddy!\n");
		printf ("Error = %s\n", strerror (errno));
		fflush (stdout);
	}
	//okay, we should be connected now...
}

void closeVariableSocket ()
{
  if (variableSocket != -1)
		close (variableSocket);
}

void getVariable (const char *name, char *buffer, int buflen)
{
  int n;
	//some initial tests
	if (!name) return;
	if (!buffer) return;
	if (buflen <= 0) return;
	//alocate memory for request-string
	n = strlen (name);
	char *request = (char *) malloc (6 + n);
	//build a request
	strcpy (request, "get ");
	strcat (request, name);
	strcat (request, "\n");
	//send it and receive the result
	sendAndReceive (request, buffer, buflen);
	//free up memory used by the request-string
	free (request);
}

char setVariable (const char *name, const char *value)
{
  int n1, n2;
	//some initial tests
  if (!name) return 0;
	if (!value) return 0;
	//alocate memory for request-string
	n1 = strlen (name);
	n2 = strlen (value);
	char *request = (char *) malloc (7 + n1 + n2);
	//build a request
	strcpy (request, "set ");
	strcat (request, name);
	strcat (request, " ");
	strcat (request, value);
	strcat (request, "\n");
	//send it and receive the result
	sendAndReceive (request, internalBuffer, 51);
	//free up memory used by the request-string
	free (request);
	
	if (strncmp (internalBuffer, "OK", 2) == 0)
		return 1;  //success
	return 0;  //failure
}

char unsetVariable (const char *name)
{
  return simpleCommand ("unset", name);
}

char incVariable (const char *name, int delta)
{
  int n1, n2;
	//some initial tests
  if (!name) return 0;
	if (delta <= 0) return 0;
	//alocate memory for request-string
	n1 = strlen (name);
	n2 = 10;
	char *request = (char *) malloc (7 + n1 + n2);
	char *value = (char *) malloc (10);
	sprintf (value, "%d", delta);
	//build a request
	strcpy (request, "inc ");
	strcat (request, name);
	strcat (request, " ");
	strcat (request, value);
	strcat (request, "\n");
	//send it and receive the result
	sendAndReceive (request, internalBuffer, 51);
	//free up memory used by the request-string
	free (request);

	if (strncmp (internalBuffer, "OK", 2) == 0)
		return 1;  //success
	return 0;  //failure
}

char decVariable (const char *name, int delta)
{
  int n1, n2;
	//some initial tests
  if (!name) return 0;
	if (delta <= 0) return 0;
	//alocate memory for request-string
	n1 = strlen (name);
	n2 = 10;
	char *request = (char *) malloc (7 + n1 + n2);
	char *value = (char *) malloc (10);
	sprintf (value, "%d", delta);
	//build a request
	strcpy (request, "dec ");
	strcat (request, name);
	strcat (request, " ");
	strcat (request, value);
	strcat (request, "\n");
	//send it and receive the result
	sendAndReceive (request, internalBuffer, 51);
	//free up memory used by the request-string
	free (request);

	if (strncmp (internalBuffer, "OK", 2) == 0)
		return 1;  //success
	return 0;  //failure
}

char provideResource (const char *name)
{
  return simpleCommand ("provide", name);
}

char requestResource (const char *name)
{
  return simpleCommand ("request", name);
}

char lockVariable (const char *name)
{
  return simpleCommand ("lock", name);
}

void unlockVariable (const char *name)
{
  simpleCommand ("unlock", name);
}

char simpleCommand (const char *command, const char *name)
{
  int n,k;
	//some initial tests
  if (!name) return 0;
	//alocate memory for request-string
	n = strlen (name);
	k = strlen (command);
	char *request = (char *) malloc (3 + k + n);
	//build a request
	strcpy (request, command);
	strcat (request, " ");
	strcat (request, name);
	strcat (request, "\n");
	//send it and receive the result
	sendAndReceive (request, internalBuffer, 51);
	//free up memory used by the request-string
	free (request);

	if (strncmp (internalBuffer, "OK", 2) == 0)
		return 1;  //success
	return 0;  //failure
}

void sendAndReceive (const char *sendthis, char *buffer, int buflen)
{
  int n;
	int tobewritten, remaining;
	const char *sendme;
	char myBuf[101];
	char gotNewLine = 0;
	buffer[0] = '\0';
	
	if (variableSocket == -1)
		return;
  tobewritten = strlen (sendthis);
	if (tobewritten == 0)
		return;
	//pointer to data that hasn't been written yet
	sendme = sendthis;
	//write it all
	while (tobewritten > 0)
	{
		n = write (variableSocket, sendme, strlen (sendme));
		tobewritten -= n;
		if (n == -1)
			//TODO: some error reporting
			return;
		//shift the pointer
		sendme += n;
	}
	
	//read reply from KMuddy
	remaining = buflen;  //buffer needs to have one extra byte for \0
	
	while (!gotNewLine)
	{
		n = read (variableSocket, myBuf, 100);
		
		if (n == -1) //bah!
			return;
		myBuf[n] = '\0';
		if (remaining)
		{
			strncat (buffer, myBuf, remaining);
			remaining -= n;
			if (remaining < 0)
				remaining = 0;
			n = strlen (buffer);
			if (n)
				if (buffer[n - 1] == '\n')
				{
					gotNewLine = 1;
					buffer[n - 1] = '\0';  //don't return that newline
				}
		}
	}
}


